@@ -24,11 +24,11 @@ script. Note that this requires you to have zxpy installed globally.
 import ast
 import code
 import inspect
-import readline
 import subprocess
+import shlex
 import sys
 import traceback
-from typing import Optional, Union
+from typing import Literal, Optional, Tuple, Union, overload
 
 
 def cli() -> None:
@@ -46,15 +46,22 @@ def cli() -> None:
     # Remove zxpy executable from argv
     sys.argv = sys.argv[1:]
 
-    filenames = sys.argv
-    if not filenames:
+    if len(sys.argv) == 0:
         setup_zxpy_repl()
         return
 
-    for filename in filenames:
-        with open(filename) as file:
-            module = ast.parse(file.read())
-            run_zxpy(filename, module)
+    filename = sys.argv[0]
+    with open(filename) as file:
+        module = ast.parse(file.read())
+        run_zxpy(filename, module)
+
+
+@overload
+def run_shell(command: str) -> str: ...
+@overload
+def run_shell(command: str, print_it: Literal[False]) -> str: ...
+@overload
+def run_shell(command: str, print_it: Literal[True]) -> None: ...
 
 
 def run_shell(command: str, print_it: bool = False) -> Optional[str]:
@@ -81,10 +88,37 @@ def run_shell(command: str, print_it: bool = False) -> Optional[str]:
     return None
 
 
+def shlex_quote(string: str) -> str:
+    """Simple wrapper for shlex.quote"""
+    return shlex.quote(string)
+
+
+def run_shell_alternate(command: str) -> Tuple[str, str, int]:
+    """Like run_shell but returns 3 values: stdout, stderr and return code"""
+    process = subprocess.Popen(
+        command,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        shell=True,
+    )
+    process.wait()
+    assert process.stdout is not None
+    assert process.stderr is not None
+    assert process.returncode is not None
+
+    return (
+        process.stdout.read().decode(),
+        process.stderr.read().decode(),
+        process.returncode,
+    )
+
+
 def run_zxpy(filename: str, module: ast.Module) -> None:
     """Runs zxpy on a given file"""
     patch_shell_commands(module)
-    exec(compile(module, filename, mode='exec'))
+    code = compile(module, filename, mode='exec')
+    # Passing `__name__` in locals() to avoid overriding any globals
+    exec(code, None, {'__name__': '__main__'})
 
 
 def patch_shell_commands(module: Union[ast.Module, ast.Interactive]) -> None:
@@ -98,19 +132,41 @@ def patch_shell_commands(module: Union[ast.Module, ast.Interactive]) -> None:
     ast.fix_missing_locations(module)
 
 
+def quote_fstring_args(fstring: ast.JoinedStr) -> None:
+    for index, node in enumerate(fstring.values):
+        if isinstance(node, ast.FormattedValue):
+            fstring.values[index] = ast.Call(
+                func=ast.Name(id='shlex_quote', ctx=ast.Load()),
+                args=[node],
+                keywords=[],
+            )
+
+
 class ShellRunner(ast.NodeTransformer):
     """Replaces the ~'...' syntax with run_shell(...)"""
     @staticmethod
-    def modify_expr(expr: ast.expr) -> ast.expr:
+    def modify_expr(
+            expr: ast.expr,
+            return_stderr_and_returncode: bool = False,
+    ) -> ast.expr:
         if (
             isinstance(expr, ast.UnaryOp)
             and isinstance(expr.op, ast.Invert)
             and isinstance(expr.operand, (ast.Str, ast.JoinedStr))
         ):
+            if isinstance(expr.operand, ast.JoinedStr):
+                quote_fstring_args(expr.operand)
+
+            function_name = (
+                'run_shell_alternate'
+                if return_stderr_and_returncode
+                else 'run_shell'
+            )
+
             return ast.Call(
-                func=ast.Name(id='run_shell', ctx=ast.Load()),
+                func=ast.Name(id=function_name, ctx=ast.Load()),
                 args=[expr.operand],
-                keywords=[]
+                keywords=[],
             )
 
         return expr
@@ -121,7 +177,11 @@ class ShellRunner(ast.NodeTransformer):
         return expr
 
     def visit_Assign(self, assign: ast.Assign) -> ast.Assign:
-        assign.value = self.modify_expr(assign.value)
+        assign.value = self.modify_expr(
+            assign.value,
+            return_stderr_and_returncode=isinstance(assign.targets[0], ast.Tuple),
+        )
+
         super().generic_visit(assign)
         return assign
 
@@ -162,9 +222,7 @@ def print_shell_outputs(statement: ast.stmt) -> None:
 
 
 def setup_zxpy_repl() -> None:
-    """
-    Sets up a zxpy interactive session.
-    """
+    """Sets up a zxpy interactive session"""
     print("zxpy shell")
     print("Python", sys.version)
     print()
@@ -191,33 +249,35 @@ def install() -> None:
         locals().update(parent_locals)
 
     # For tab completion and arrow key support
-    readline.parse_and_bind("tab: complete")
+    if sys.platform != 'win32':
+        import readline
+        readline.parse_and_bind("tab: complete")
 
     command = ''
     continued_command = False
     while True:
         try:
-            if continued_command:
-                command += '\n'
-            else:
-                command = ''
-
             prompt = '... ' if continued_command else '>>> '
             new_input = input(prompt)
-
-            if new_input != '':
-                command += new_input
-            else:
-                continued_command = False
-
         except KeyboardInterrupt:
             print()
             continue
-
         except EOFError:
             print()
             sys.exit(0)
 
+        # TODO: refactor the next 10 lines.
+        # probably move command = '...' stuff somewhere else
+        if continued_command:
+            command += '\n'
+        else:
+            command = ''
+
+        if new_input != '':
+            command += new_input
+        else:
+            continued_command = False
+
         if continued_command:
             continue
 
